% controls BPMmodel and other files
%
% February 2011
% Mariano Beguerisse


clear all
tic
%%%% Setup %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% call setup with all options of the specific problem
% Objective function should be given in objFun.m

algorithmSetup;



%%%% Fit parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if fitData == 1
    % RUN ITERATIONS
    for iter=iterStart:numRuns
        % Create folder for this run
        runFolderName = [modelFolderName '/' 'Run' num2str(iter)];
        if exist(runFolderName)==0
            fprintf('\nMaking %s\n', runFolderName)
            mkdir(runFolderName);
        end
        
        
        prior = 0;
        priorErr = [];
        Etot = [];
        % If there is a previous iteration, load results  
        if iter>1 
            % Get prior of parameters from previous run
            previousRun = [modelFolderName '/' 'Run' num2str(iter-1)];
            [Etot] = loadResults([ModName '*.mat'], previousRun);
            if length(Etot) > 0
                prior = Etot(:, 2:end);
                priorErr = Etot(:, 1);
                % Update historical prior
                priorH(1, :) = ...
                    min([Etot(:, 2:end); priorH(1, :)]);
                priorH(2, :) = ...
                    max([Etot(:, 2:end); priorH(2, :)]);
                
            else
                Etot = [];
            end
        end
        
        % Fit parameters
        fprintf('\n\nFit data. Iteration: %i\n', iter)
        
        % Results of this fitting round stored in ERun
        [ERun S history] = fitParams(iter, ModName, X0, N, Data, ...
                                   prior, priorH, maxIterFit, ...
                                   numSavErr, numDec, options, poolSize);
                
        
        % Cumulative History in E (merges ERun and Etot), 
        % sorted from best 1st Row
        % to worst. E(1,1) is the error of the best parameters
        % E(1, 2:end) is the best parameter set.
        E = sortrows([ERun; Etot]);
        
        % Cull the worst parameters
        E = E(1:numSavErr, :);
        
        % Get the posterior including new resuts
        posterior = E(:, 2:end);
        postErr = E(:, 1);
        
        % Perform param distribution and error convergence check
        parH = convergenceTest(prior, posterior);
        errH = convergenceTest(priorErr, postErr);
        
        fprintf('\nParameter distribution convergence test:\n')
        fprintf('parH = [%s]', num2str(parH));
        fprintf('\nError distribution convergence test:\n')
        fprintf('errH = %i\n', errH);
        
        fprintf('\nBest param so far:\n%s\n', num2str(E(1,2:end)));
        fprintf('\nBest error so far:\n%f\n', E(1,1));
        
        % Save results in file
        fileName = [runFolderName '/', ModName, ...
                    datestr(now, 'ddmmmHHMMss'), '.mat'];
        save(fileName, 'E', 'ERun', 'S', 'history', 'prior', 'priorH');
        fprintf('\nSaved %s\n', fileName)
        toc
    end
    toc
end





%%%% Explore results %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if exploreData == 1
    expFolderName = [modelFolderName '/' 'Run' num2str(expIter)];
    [Etot Stot hst] = loadResults([ModName '*.mat'], expFolderName);
    params = Etot(1, 2:end);
    exploreE(Etot, r, ModName, X0, Data, varNames, paramNames);
end

    
warning on all;
