
%%%%%%  GLOBAL OPTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Reset the random number generator seed
dv = datevec(now); 
newSeed = 10000*dv(end);
%fprintf('\nNew random number seed: %i\n', newSeed)
rand('state', newSeed); 

% old version
%RandStream.setDefaultStream(RandStream('mt19937ar', 'seed', newSeed));


% Parameters and options of the Nelder-Mead routine
% These can be changed to optimise performance, see
% MATLAB's optimset options
maxEvals = 300;
maxIter = 300;
TolFun = 1e-4;
TolX = 1e-4;
options = optimset('Display', 'off', 'MaxFunEvals', maxEvals, ...
                   'MaxIter', maxIter, 'TolFun', TolFun, ...
                   'TolX', TolX); 


% Number of paralell workers
poolSize = 8;



%%%% Read the Data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% CSV file Summary sheet contains all the data of experiments in
% wt plants.

% fake data with a= 240 and b= 0.15 + 25*randn
% Matrix of noisy data
 Data = textread('BPMDataNoise.csv', '','delimiter', ',', ...
                 'headerlines', 1,'emptyvalue', NaN);

% Matrix of data with no noise, error should be zero
%Data = textread('BPMData.csv', '','delimiter', ',', ...
%                 'headerlines', 1,'emptyvalue', NaN);

sizeData = size(Data, 2);


%%%% Model information %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ModName = 'BPMmodel';    % Name of the ODE model file
numVar = 3;              % number of variables
N = 2;                   % number of parameters to fit
X0 = zeros(numVar,1);     % initial condition of the ODE

varNames = {'R', 'E', 'P'};
paramNames = {'\alpha' '\beta'};

% Folder name to store results:
modelFolderName = ['results' ModName];
 
% Prior that will also be the historical prior
% This is the initial distrubution to sample for
% each parameter.
% Change accordingly

% Uniform (0, 100) for alpha and (0, 5) for beta
priorH = [zeros(1,N); [100 5]]; 

% This is quite extreme, but it works (takes longer): Uniform (0, 100) 
%priorH = [zeros(1,N); 100*ones(1,N)]; 


%%%% Tasks %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Number of iterations and which iteration to start from
% when resuming work
numRuns = 10;                       % Number of times the prior is updated.
iterStart = 1;       
maxIterFit = 250;                   %number of times Nelder-Mead is run
p = 15;                             %Percent of results to keep in E
numSavErr = ceil(maxIterFit*p/100); %number of entries to be used
                                    %in the prior
numDec = 10;                        % number of decimals to compare

% What to do:
fitData = 1;          % 1 to fit parameters for BPM, 0 to skip
exploreData = 1;      % 1 to see results, 0 to skip

expIter = numRuns;    % What iteration to explore when finished
r = 1;                % Row to explore (1 is best result, 2 second
                      % best, etc)